/* wipe
 * $Id: rc6.c,v 1.1.1.1 2002/11/25 23:59:49 berke Exp $
 * by Berke Durak
 * Author may be contacted at 'echo berke1altern2org|tr 12 @.'
 *
 * URL for wipe: http://perso.wanadoo.fr/berke/wipe/
 *
 * Straightforward unoptimised naive simple and stupid, as well as dumb
 * RC6 implementation.
 *
 */

#ifdef RC6_ENABLED
#include "rc6.h"

#define ROL(x,y) (((x) << (y)) | ((x) >> (32 - (y))))
#define ROR(x,y) (((x) >> (y)) | ((x) << (32 - (y))))

#define max(x,y) ((x>y)?x:y)

#if REVERSEBYTEORDER
#define h(x) ((x<<24) | ((0xff00 & x)<<16) | ((x >> 8) & 0xff00) | (x >> 24))
#else
#define h(x) x
#endif

void rc6_SetupKey (u8 k[RC6_B], struct rc6_KeySchedule *ks)
{
  u32 *s;
  u32 a, b;
  u32 l[RC6_C];
  int i, j, v, w;

  for (i = 0; i<RC6_C; i++) {
    a = (k[i*4 + 0] << 24) |
      (k[i*4 + 1] << 16) |
      (k[i*4 + 2] <<  8) |
      (k[i*4 + 3]);
    l[i] = a;
  }

  s = ks->s;

  s[0] = RC6_PW;
  for (i = 1; i < 2*RC6_R+4; i++) {
    s[i] = s[i-1] + RC6_QW;
  }

  v = 3 * max (RC6_C, 2*RC6_R + 4);
  i = j = 0;
  a = b = 0;

  for (w = 0; w < v; w++) {
    a = s[i] = ROL (s[i] + a + b, 3);
    b = l[j] = ROL (l[j] + a + b, (a + b) & RC6_WMSK);
    i ++; if (i == 2*RC6_R + 4) i = 0;
    j ++; if (j == RC6_C) j = 0;

  }
}

void rc6_Encrypt (u32 src[4], u32 dst[4], struct rc6_KeySchedule *ks)
{
  int i;
  u32 *s;
  u32 a, b, c, d;

  a = h (src[0]); b = h (src[1]); c = h (src[2]); d = h (src[3]);

  s = ks->s;

  b += *(s++); d += *(s++);

  for (i = 0; i<RC6_R; i++) {
    u32 t, u, tt;

    t = ROL ((b * (2*b + 1)), RC6_LGW);
    u = ROL ((d * (2*d + 1)), RC6_LGW);
    a = ROL (a ^ t, RC6_WMSK & u) + s[0];
    c = ROL (c ^ u, RC6_WMSK & t) + s[1];

    s += 2;
    tt = a; a = b; b = c; c = d; d = tt;

  }
  a = a + *(s++); c = c + *s;
  dst[0] = h (a); dst[1] = h (b); dst[2] = h (c); dst[3] = h (d);
}

void rc6_Decrypt (u32 src[4], u32 dst[4], struct rc6_KeySchedule *ks)
{
  int i;
  u32 *s;
  u32 a, b, c, d;

  a = h (src[0]); b = h (src[1]); c = h (src[2]); d = h (src[3]);

  s = ks->s + 2*RC6_R + 3;

  c -= *(s--); a -= *(s--);

  for (i = 0; i<RC6_R; i++) {
    u32 t, u, tt;

    tt = d; d = c; c = b; b = a; a = tt; 

    u = ROL ((d * (2*d + 1)), RC6_LGW);
    t = ROL ((b * (2*b + 1)), RC6_LGW);
    c = ROR (c - s[0 ], RC6_WMSK & t) ^ u;
    a = ROR (a - s[-1], RC6_WMSK & u) ^ t;

    s -= 2;
  }
  d = d - *(s--); b = b - *s;

  dst[0] = h (a); dst[1] = h (b); dst[2] = h (c); dst[3] = h (d);
}
#endif /* RC6_ENABLED */

/* vim:set sw=4:set ts=8: */
